@@ -62,7 +62,7 @@
     this pattern is straightforward, once the decipher_expression is
     properly implemented).
 
-    Copyright (c) 2012 European Space Agency
+    Copyright (c) 2012-2013 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -184,7 +184,7 @@ def output_statement(output):
         proc, out_sig = None, None
         try:
             out_sig, = [sig for sig in OUT_SIGNALS
-                        if sig['name'] == signal_name.lower()]
+                        if sig['name'].lower() == signal_name.lower()]
         except ValueError:
             # Not an output, try if it is an external procedure
             try:
@@ -213,8 +213,8 @@ def output_statement(output):
                 # Create a temporary variable for input parameters only
                 if param_direction == 'in':
                     tmp_id = out['tmpVars'][idx]
-                    local_decl.append('tmp{idx} : aliased asn1Scc{oType};'.format(
-                        idx=tmp_id, oType=param_type))
+                    local_decl.append('tmp{idx} : aliased asn1Scc{oType};'
+                                      .format(idx=tmp_id, oType=param_type))
                     code.append('tmp{idx} := {p_id};'.format(
                         idx=tmp_id, p_id=p_id))
                     list_of_params.append("tmp{idx}'access".
@@ -356,7 +356,7 @@ def decipher_primary_id(primaryId):
                     # needed to build the Ada expression
                     # (Won't work for embedded types - FIXME)
                     exp_typename = (exp.exprType.get('ReferencedTypeName') or
-                            exp.exprType['Kind']).replace('-', '_') 
+                            exp.exprType['Kind']).replace('-', '_')
                     if exp_type['Kind'] != 'ChoiceType':
                         error = '{} is not a CHOICE'.format(exp.inputString)
                         LOG.error(error)
@@ -389,8 +389,8 @@ def decipher_expression(expr):
     ada_string = ''
     local_decl = []
     if expr.kind == 'primary':
-        if (expr.var.primType == None
-                or expr.var.primType['Kind'] != 'ReferenceType'):
+        if (expr.var.primType == None):
+               # or expr.var.primType.get('Kind') != 'ReferenceType'):
             # Populate the expression type to the field if it was not set
             # It can be the case if the type definition is embedded
             # in a SEQUENCE - in that case the ASN.1 compiler creates an
@@ -410,11 +410,87 @@ def decipher_expression(expr):
         local_decl.extend(left_local)
         local_decl.extend(right_local)
     elif expr.kind == 'append':
-        # TODO (Append item to a list)
-        pass
+        # Append item to a list - Octet string (TODO: sequence of)
+        basic_type_expr = find_basic_type(expr.exprType)
+        # We can do a length check if both strings are literals
+        if(expr.right.kind == 'primary'
+           and expr.right.var.kind == 'stringLiteral'
+           and expr.left.kind == 'primary'
+           and expr.left.var.kind == 'stringLiteral'
+           and len(expr.right.var.stringLiteral[1:-1]) +
+           len(expr.left.var.stringLiteral[1:-1]) > basic_type_expr['Max']):
+            raise ValueError('String concatenation exceeds container length: '
+                    'length(' + expr.left.var.stringLiteral[1:-1] +
+                    expr.right.var.stringLiteral[1:-1] + ') > ' +
+                    str(basic_type_expr['Max']))
+
+        left_stmts, left_str, left_local = decipher_expression(expr.left)
+        right_stmts, right_str, right_local = decipher_expression(expr.right)
+        stmts.extend(left_stmts)
+        stmts.extend(right_stmts)
+        local_decl.extend(left_local)
+        local_decl.extend(right_local)
+        # Declare a temporary variable to hold the result of the append
+        ada_string = 'tmp{}'.format(expr.tmpVar)
+        local_decl.append('{tmp} : aliased asn1Scc{eType};'.format(
+                        tmp=ada_string,
+                        eType=expr.exprType['ReferencedTypeName']))
+
+        # If right or left is raw, declare a temporary variable for it, too
+        for sexp, sid in zip((expr.right, expr.left), (right_str, left_str)):
+            if sexp.is_raw():
+                local_decl.append('tmp{idx} : aliased asn1Scc{eType};'.format(
+                        idx=sexp.var.tmpVar,
+                        eType=sexp.exprType['ReferencedTypeName']))
+                stmts.append('tmp{idx} := {s_id};'.format(
+                    idx=sexp.var.tmpVar, s_id=sid))
+                sexp.sid = 'tmp' + str(sexp.var.tmpVar)
+                # Length of raw string - update for sequence of
+                if sexp.var.kind == 'stringLiteral':
+                    sexp.slen = len(sexp.var.stringLiteral[1:-1])
+                elif sexp.var.kind == 'emptyString':
+                    sexp.slen = 0
+                elif sexp.var.kind == 'sequenceOf':
+                    sexp.slen = len(sexp.var.sequenceOf)
+                else:
+                    raise TypeError('Not a string/Sequence in APPEND')
+            else:
+                sexp.sid = sid
+                basic = find_basic_type(sexp.exprType)
+                if basic['Min'] == basic['Max']:
+                    # Fixed-size string
+                    sexp.slen = basic['Max']
+                else:
+                    # Variable-size types have a Length field
+                    sexp.slen = '{}.Length'.format(sexp.sid)
+        stmts.append('{res}.Data(1..{l1}) := {lid}.Data(1..{l1});'.format(
+                    res=ada_string, l1=expr.left.slen, lid=expr.left.sid))
+        stmts.append('{res}.Data({l1}+1..{l1}+{l2}) := {rid}.Data(1..{l2});'
+                    .format(res=ada_string, l1=expr.left.slen,
+                    rid=expr.right.sid, l2=expr.right.slen))
+        stmts.append('{res}.Length := {l1} + {l2};'.format(
+                    res=ada_string, l1=expr.left.slen, l2=expr.right.slen))
+
     elif expr.kind == 'in':
-        # TODO (Check if item is in a list/set)
-        pass
+        # Check if item is in a SEQUENCE OF
+        # Temporary variable needed to hold the test result
+        ada_string = 'tmp{}'.format(expr.tmpVar)
+        local_decl.append('{} : BOOLEAN := False;'.format(ada_string))
+        left_stmts, left_str, left_local = decipher_expression(expr.left)
+        right_stmts, right_str, right_local = decipher_expression(expr.right)
+        stmts.extend(left_stmts)
+        stmts.extend(right_stmts)
+        local_decl.extend(left_local)
+        local_decl.extend(right_local)
+        stmts.append("in_loop_{}:".format(ada_string))
+        stmts.append("for elem in {}.Data'Range loop".format(left_str))
+        stmts.append("if {container}.Data(elem) = {pattern} then".format
+                (container=left_str, pattern=right_str))
+        stmts.append("{} := True;".format(ada_string))
+        stmts.append("end if;")
+        stmts.append("exit in_loop_{tmp} when {tmp} = True;".format(tmp=ada_string))
+        stmts.append("end loop in_loop_{};".format(ada_string))
+
     elif expr.kind == 'rem':
         # TODO (Remove item from a set?)
         pass
@@ -434,7 +510,9 @@ def decipher_primary(primary):
         # (note: in C we would need to read EnumID in the type definition)
         ada_string = 'asn1Scc{enumVal}'.format(enumVal=primary.primaryId[0])
     elif primary.kind == 'choiceDeterminant':
-        ada_string = '{choice}_PRESENT'.format(choice=primary.primaryId[0])
+        enumerant = primary.primaryId[0]
+        ada_string = primary.primType['EnumValues'][enumerant]['EnumID']
+        #ada_string = '{choice}_PRESENT'.format(choice=primary.primaryId[0])
     elif primary.kind in ('numericalValue', 'booleanValue'):
         ada_string = primary.primaryId[0]
     elif primary.kind == 'sequence':
@@ -450,7 +528,19 @@ def decipher_primary(primary):
         ada_string = 'asn1Scc{typeRef}_Init'.format(
               typeRef=primary.primType['ReferencedTypeName'].replace('-', '_'))
     elif primary.kind == 'stringLiteral':
-        pass
+        basic_type = find_basic_type(primary.primType)
+        if basic_type['Kind'].endswith('StringType'):
+            # If user put a literal string to fill an Octet string,
+            # then convert the string to an array of unsigned_8 integers
+            # as expected by the Ada type corresponding to Octet String
+            unsigned_8 = [str(ord(val)) for val in primary.stringLiteral[1:-1]]
+            ada_string = '(Data => (' + ', '.join(
+                                                 unsigned_8) + ', others => 0)'
+            if basic_type['Min'] != basic_type['Max']:
+                # Non-fixed string size -> add Length field
+                ada_string += ', Length => {}'.format(
+                                        str(len(primary.stringLiteral[1:-1])))
+            ada_string += ')'
     elif primary.kind == 'ifThenElse':
         stmts, ada_string, local_decl = decipher_if_then_else(
                 primary.ifThenElse, primary.primType)
@@ -716,20 +806,19 @@ def inner_procedure(proc):
         params = []
         for fpar in proc.fpar:
             params.append('l_{name}: in{out} asn1Scc{ptype}'.format(
-                    name = fpar.get('name'),
-                    out = ' out' if fpar.get('direction') == 'out' else '',
-                    ptype = fpar.get('type')))
+                    name=fpar.get('name'),
+                    out=' out' if fpar.get('direction') == 'out' else '',
+                    ptype=fpar.get('type')))
         pi_header += ';'.join(params)
         pi_header += ')'
 
     local_decl.append(pi_header + ';')
 
-
     # Generate the code for the procedure itself (unless it's external):
     # local variables and code of the START transition
     if not proc.external:
         # Recursively generate the code for inner-defined procedures
-        for inner_proc in proc.inner_procedures:
+        for inner_proc in proc.content.inner_procedures:
             inner_code, inner_local = inner_procedure(inner_proc)
             local_decl.extend(inner_local)
             code.extend(inner_code)
@@ -737,10 +826,10 @@ def inner_procedure(proc):
         for var_name, var_type in proc.variables.viewitems():
             code.append('l_{name} : asn1Scc{sort};'.format(
                 name=var_name, sort=var_type))
-        tr_code, tr_decl = transition(proc.start.transition)
+        tr_code, tr_decl = transition(proc.content.start.transition)
         # Generate code for the floating labels
         code_labels = []
-        for label in proc.floating_labels:
+        for label in proc.content.floating_labels:
             code_label, label_decl = floating_label(label)
             code_labels.extend(code_label)
             tr_decl.extend(label_decl)
@@ -765,7 +854,7 @@ def format_ada_code(stmts):
         if elems and elems[-1] == 'case;':
             indent = max(indent - 1, 0)
         yield indent_pattern * indent + line
-        if elems and elems[-1] in ('is', 'then'):
+        if elems and elems[-1] in ('is', 'then', 'loop'):
             indent += 1
         if elems and elems[0] in ('begin', 'case', 'else', 'when'):
             indent += 1
@@ -791,7 +880,7 @@ def generate(process):
 
     #PROCEDURES.update({p.lower(): defn for p, defn in
     #        process.procedures.viewitems()})
-    INNER_PROCEDURES.extend(process.inner_procedures)
+    INNER_PROCEDURES.extend(process.content.inner_procedures)
 
     LOG.info('Generating Ada code for process ' + str(process_name))
 
@@ -810,7 +899,6 @@ def generate(process):
     # Add the declaration of the runTransition procedure
     process_level_decl.append('procedure runTransition(trId: Integer);')
 
-
     # Generate the code of the start transition:
     start_transition = ['begin']
     start_transition.append('runTransition(0);')
@@ -818,7 +906,7 @@ def generate(process):
     mapping = {}
     # Generate the code for the transitions in a mapping input-state
     input_signals = [sig['name'] for sig in process.input_signals]
-    for input_signal in input_signals: #process.inputSignals.viewkeys():
+    for input_signal in input_signals:
         mapping[input_signal] = {}
         for state_name, input_symbols in process.mapping.viewitems():
             if state_name != 'START':
@@ -828,9 +916,12 @@ def generate(process):
                         mapping[input_signal][state_name] = i
 
     # Generate the TASTE template
-    asn1_modules = '\n'.join(['with {dv};\nuse {dv};'.format(
-        dv=dv.replace('-', '_'))
-        for dv in process.asn1Modules])
+    try:
+        asn1_modules = '\n'.join(['with {dv};\nuse {dv};'.format(
+            dv=dv.replace('-', '_'))
+            for dv in process.asn1Modules])
+    except TypeError:
+        asn1_modules = '--  No ASN.1 data types are used in this model'
     taste_template = ['''\
 -- This file was generated automatically: DO NOT MODIFY IT !
 
@@ -859,14 +950,14 @@ package {process_name} is'''.format(process_name=process_name,
 
     # Generate the the code of the procedures
     inner_procedures_code = []
-    for proc in process.inner_procedures:
+    for proc in process.content.inner_procedures:
         proc_code, proc_local = inner_procedure(proc)
         process_level_decl.extend(proc_local)
         inner_procedures_code.extend(proc_code)
 
     # Generate the code for the process-level variable declarations
     taste_template.extend(process_level_decl)
-    
+
     # Add the code of the procedures definitions
     taste_template.extend(inner_procedures_code)
 
@@ -882,7 +973,7 @@ package {process_name} is'''.format(process_name=process_name,
                 pName=param_name, pType=signal['type'])
 
         # Add declaration of the provided interface in the .ads file
-        ads_template.append('-- Provided interface "'+ signal['name'] + '"')
+        ads_template.append('-- Provided interface "' + signal['name'] + '"')
         ads_template.append(pi_header + ';')
 
         pi_header += ' is'
@@ -912,7 +1003,8 @@ package {process_name} is'''.format(process_name=process_name,
         taste_template.append('when others =>')
         taste_template.append('null;')
         taste_template.append('end case;')
-        taste_template.append('end {sig_name};'.format(sig_name=signal['name']))
+        taste_template.append('end {sig_name};'.format(
+                                                      sig_name=signal['name']))
         taste_template.append('\n')
 
     # for the .ads file, generate the declaration of the required interfaces
@@ -924,7 +1016,7 @@ package {process_name} is'''.format(process_name=process_name,
         if 'type' in signal:
             ri_header += '({pName}: access asn1Scc{pType});'.format(
                 pName=param_name, pType=signal['type'])
-        ads_template.append('-- Required interface "'+ signal['name'] + '"')
+        ads_template.append('-- Required interface "' + signal['name'] + '"')
         ads_template.append(ri_header)
         ads_template.append('pragma import(C, {sig}, "{proc}_RI_{sig}");'
                 .format(sig=signal['name'], proc=process_name))
@@ -938,7 +1030,7 @@ package {process_name} is'''.format(process_name=process_name,
                 par=param))
         if params:
             ri_header += '(' + ';'.join(params) + ');'
-        ads_template.append('-- Sync required interface "'+ proc.inputString)
+        ads_template.append('-- Sync required interface "' + proc.inputString)
         ads_template.append(ri_header)
         ads_template.append('pragma import(C, {sig}, "{proc}_RI_{sig}");'
                 .format(sig=proc.inputString, proc=process_name))
@@ -955,7 +1047,7 @@ package {process_name} is'''.format(process_name=process_name,
 
     # Generate code for the floating labels
     code_labels = []
-    for label in process.floating_labels:
+    for label in process.content.floating_labels:
         code_label, label_decl = floating_label(label)
         code_labels.extend(code_label)
         local_decl_transitions.extend(label_decl)
@@ -991,7 +1083,7 @@ package {process_name} is'''.format(process_name=process_name,
     taste_template.extend(start_transition)
     taste_template.append('end {process_name};'
             .format(process_name=process_name))
-    
+
     ads_template.append('end {process_name};'
             .format(process_name=process_name))
 
