@@ -5,7 +5,7 @@
 
     SDL is the Specification and Description Language (Z100 standard from ITU)
 
-    Copyright (c) 2012 European Space Agency
+    Copyright (c) 2012-2013 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -21,14 +21,15 @@
 
 import math
 import PySide
+from PySide import QtGui, QtCore
 import genericSymbols
 import logging
 try:
     import pygraphviz as dotgraph
 except ImportError:
-    print('pygraphviz not found. Statechart modulew will not work\n'
+    print('pygraphviz not found. Statechart module will not work\n'
             'If you are using Debian (or Ubuntu) you can install it from\n'
-            'the official depots: sudo apt-get install python-pygraphviz')
+            'the official repos: sudo apt-get install python-pygraphviz')
 
 RENDER_DPI = {'X': 93.0, 'Y': 95.0}
 G_SYMBOLS = set()
@@ -36,12 +37,15 @@ EDGES = []
 
 LOG = logging.getLogger(__name__)
 
+
 # pylint: disable=R0904
 class Record(genericSymbols.HorizontalSymbol, object):
     ''' Graphviz node - it is floating and has no parent'''
-    allowed_followers = ['Record', 'Diamond']
-    textbox_alignment = (PySide.QtCore.Qt.AlignTop
-                         | PySide.QtCore.Qt.AlignHCenter)
+    _unique_followers = []
+    _insertable_followers = ['Record', 'Diamond']
+    _terminal_followers = []
+    textbox_alignment = (QtCore.Qt.AlignTop
+                         | QtCore.Qt.AlignHCenter)
 
     def __init__(self, node, graph):
         ''' Initialization: compute the polygon shape '''
@@ -49,16 +53,16 @@ class Record(genericSymbols.HorizontalSymbol, object):
         super(Record, self).__init__(x=node['pos'][0],
                 y=node['pos'][1], text=self.name)
         self.set_shape(node['width'], node['height'])
-        self.setBrush(PySide.QtGui.QBrush(PySide.QtGui.QColor(255, 255, 202)))
+        self.setBrush(QtGui.QBrush(QtGui.QColor(255, 255, 202)))
         self.graph = graph
         if 'properties' in node:
-            property_box = PySide.QtGui.QGraphicsTextItem(self)
+            property_box = QtGui.QGraphicsTextItem(self)
             property_box.setPos(5, 30)
             property_box.setPlainText(node['properties'])
 
     def set_shape(self, width, height):
         ''' Define the polygon shape from width and height '''
-        path = PySide.QtGui.QPainterPath()
+        path = QtGui.QPainterPath()
         path.moveTo(0, 25)
         path.lineTo(width, 25)
         # check if adding mode = Qt.RelativeSize would not be better
@@ -82,15 +86,18 @@ class Record(genericSymbols.HorizontalSymbol, object):
         delta = [delta_pos.x(), delta_pos.y()]
         for edge in edges(self.scene(), self):
             if edge.edge['source'] == self:
-                edge.edge['start'] = [
-                        sum(u) for u in zip(edge.edge['spline'][0], delta)]
-                edge.edge['spline'][0] = edge.edge['start']
+                #edge.edge['start'] = [
+                #        sum(u) for u in zip(edge.edge['spline'][0], delta)]
+                #edge.edge['spline'][0] = edge.edge['start']
+                edge.end_point -= delta_pos
             if edge.edge['target'] == self:
-                edge.edge['end'] = [
-                        sum(u) for u in zip(edge.edge['end'], delta)]
-                edge.edge['spline'][-1] = edge.edge['end']
-                #[edge.edge['end'][d] + delta[d]
-                #                    for d in range(2)]
+                #edge.edge['end'] = [
+                #        sum(u) for u in zip(edge.edge['end'], delta)]
+                edge.end_point += delta_pos
+            #edge.end_point = [
+            #       sum(u) for u in zip(edge.edge['end'], delta)]
+
+                #edge.edge['spline'][-1] = edge.edge['end']
             edge.reshape()
 
     def mouse_release(self, _):
@@ -103,9 +110,11 @@ class Record(genericSymbols.HorizontalSymbol, object):
 # pylint: disable=R0904
 class Point(genericSymbols.HorizontalSymbol, object):
     ''' Graphviz point node - used for START transition'''
-    allowed_followers = ['Record', 'Diamond']
-    textbox_alignment = (PySide.QtCore.Qt.AlignTop
-                         | PySide.QtCore.Qt.AlignHCenter)
+    _unique_followers = []
+    _insertable_followers = ['Record', 'Diamond']
+    _terminal_followers = []
+    textbox_alignment = (QtCore.Qt.AlignTop
+                         | QtCore.Qt.AlignHCenter)
 
     def __init__(self, node, graph):
         ''' Initialization: compute the polygon shape '''
@@ -113,16 +122,16 @@ class Point(genericSymbols.HorizontalSymbol, object):
         super(Point, self).__init__(x=node['pos'][0],
                 y=node['pos'][1], text='')
         self.set_shape(node['width'], node['height'])
-        self.setBrush(PySide.QtGui.QBrush(PySide.QtCore.Qt.black))
+        self.setBrush(QtGui.QBrush(QtCore.Qt.black))
         self.graph = graph
 
     def set_shape(self, width, height):
         ''' Define the polygon shape from width and height '''
-        path = PySide.QtGui.QPainterPath()
+        path = QtGui.QPainterPath()
         path.addEllipse(0, 0, width, height)
         self.setPath(path)
         super(Point, self).set_shape(width, height)
-    
+
     def resize_item(self, _):
         ''' Redefine the resizing function - forbid resizing '''
         pass
@@ -134,16 +143,18 @@ class Point(genericSymbols.HorizontalSymbol, object):
 
     def mouse_release(self, _):
         ''' After moving item, ask dot to recompute the edges '''
-        #self.scene().refresh()        
+        #self.scene().refresh()
         update(self.scene())
 
 
 # pylint: disable=R0904
 class Diamond(genericSymbols.HorizontalSymbol, object):
     ''' Graphviz node - it is floating and has no parent'''
-    allowed_followers = ['Record']
-    textbox_alignment = (PySide.QtCore.Qt.AlignTop
-                         | PySide.QtCore.Qt.AlignHCenter)
+    _unique_followers = []
+    _insertable_followers = ['Record']
+    _terminal_followers = []
+    textbox_alignment = (QtCore.Qt.AlignTop
+                         | QtCore.Qt.AlignHCenter)
 
     def __init__(self, node, graph):
         ''' Initialization: compute the polygon shape '''
@@ -151,12 +162,12 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
         super(Diamond, self).__init__(x=node['pos'][0],
                 y=node['pos'][1], text='')
         self.set_shape(node['width'], node['height'])
-        #self.setBrush(PySide.QtGui.QBrush(PySide.QtCore.Qt.black))
+        #self.setBrush(QtGui.QBrush(QtCore.Qt.black))
         self.graph = graph
 
     def set_shape(self, width, height):
         ''' Define the polygon shape from width and height '''
-        path = PySide.QtGui.QPainterPath()
+        path = QtGui.QPainterPath()
         path.moveTo(width / 2, 0)
         path.lineTo(width, height / 2)
         path.lineTo(width / 2, height)
@@ -164,7 +175,7 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
         path.lineTo(width / 2, 0)
         self.setPath(path)
         super(Diamond, self).set_shape(width, height)
-    
+
     def resize_item(self, _):
         ''' Redefine the resizing function - forbid resizing '''
         pass
@@ -176,44 +187,92 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
 
     def mouse_release(self, _):
         ''' After moving item, ask dot to recompute the edges '''
-        #self.scene().refresh()        
+        #self.scene().refresh()
         update(self.scene())
 
 
+class Controlpoint(QtGui.QGraphicsPathItem, object):
+    ''' Class handling one connection control point (mouse movements) '''
+    def __init__(self, pos, parent):
+        ''' Set the original control point - with color, shape '''
+        path = QtGui.QPainterPath()
+        path.addEllipse(pos.x() - 5, pos.y() - 5, 10, 10)
+        super(Controlpoint, self).__init__(path, parent)
+        self.setPen(QtGui.QColor(50, 100, 120, 200))
+        self.setBrush(QtGui.QColor(200, 200, 210, 120))
+        self.setFlags(QtGui.QGraphicsItem.ItemIsSelectable |
+                QtGui.QGraphicsItem.ItemIsMovable)
+        self.hide()
+
+    @property
+    def center(self):
+        ''' Return the position of the center of the shape '''
+        return self.pos() + self.boundingRect().center()
+
+    def mouseMoveEvent(self, event):
+        ''' When user moves a control point, update the connection shape '''
+        super(Controlpoint, self).mouseMoveEvent(event)
+        self.parentItem().reshape()
+
+
 class Edge(genericSymbols.Connection, object):
     ''' B-spline/Bezier connection shape '''
     def __init__(self, edge, graph):
         ''' Set generic parameters from Connection class '''
+        self.text_label = None
         super(Edge, self).__init__(edge['source'], edge['target'])
         self.edge = edge
         self.graph = graph
-        #self.setBrush(PySide.QtGui.QBrush(PySide.QtGui.QColor(0, 0, 255)))
+
+        # Initialize control point coordinates
+        # Start and End points are optional - graphviz decision
+        self.start_point = (self.mapFromScene(*self.edge['start']) if
+                 self.edge.get('start') else None)
+        self.end_point = (self.mapFromScene(*self.edge['end']) if
+                 self.edge.get('end') else None)
+        self.bezier = [self.mapFromScene(*self.edge['spline'][0])]
+        # Bezier control points (groups of three points):
+        assert(len(self.edge['spline']) % 3 == 1)
+        for i in xrange(1, len(self.edge['spline']), 3):
+            self.bezier.append([Controlpoint(
+                          self.mapFromScene(*self.edge['spline'][i + j]), self)
+                          for j in range(3)])
+        # Set connection points as not visible, by default
+        self.bezier_visible = False
         self.reshape()
 
-    # pylint: disable=R0914
+    def bezier_set_visible(self, visible=True):
+        ''' Display or hide the edge control points '''
+        self.bezier_visible = visible
+        for group in self.bezier[1:]:
+            for ctrl_point in group:
+                if visible:
+                    ctrl_point.show()
+                else:
+                    ctrl_point.hide()
+        self.update()
+
+    def mousePressEvent(self, event):
+        ''' On a mouse click, display the control points '''
+        self.bezier_set_visible(True)
+
+   # pylint: disable=R0914
     def reshape(self):
         ''' Update the shape of the edge (redefined function) '''
-        path = PySide.QtGui.QPainterPath()
-        assert(len(self.edge['spline']) % 3 == 1)
-        # If there is a starting point, draw
-        # a line from it to the first curve point
-        if self.edge.get('start'):
-            path.moveTo(self.mapFromScene(*self.edge['start']))
-            path.lineTo(self.mapFromScene(*self.edge['spline'][0]))
+        path = QtGui.QPainterPath()
+        # If there is a starting point, draw a line to the first curve point
+        if self.start_point:
+            path.moveTo(self.start_point)
+            path.lineTo(self.bezier[0])
         else:
-            path.moveTo(self.mapFromScene(*self.edge['spline'][0]))
+            path.moveTo(self.bezier[0])
         # Loop over the curve points:
-        for i in xrange(1, len(self.edge['spline']), 3):
-            c_1 = self.mapFromScene(
-                    PySide.QtCore.QPointF(*self.edge['spline'][i]))
-            c_2 = self.mapFromScene(
-                    PySide.QtCore.QPointF(*self.edge['spline'][i + 1]))
-            end_point = self.mapFromScene(
-                    PySide.QtCore.QPointF(*self.edge['spline'][i + 2]))
-            path.cubicTo(c_1, c_2, end_point)
+        for group in self.bezier[1:]:
+            path.cubicTo(*[point.center for point in group])
+
         # If there is an ending point, draw a line to it
-        if self.edge.get('end'):
-            path.lineTo(self.mapFromScene(*self.edge['end']))
+        if self.end_point:
+            path.lineTo(self.end_point)
 
         # Draw the arrow head
         length = path.length()
@@ -222,15 +281,15 @@ class Edge(genericSymbols.Connection, object):
         #angle = path.angleAtPercent(percent)
         #print angle
         end_point = path.currentPosition()
-        line = PySide.QtCore.QLineF(src, end_point)
+        line = QtCore.QLineF(src, end_point)
         angle = math.acos(line.dx() / line.length())
         if line.dy() >= 0:
             angle = math.pi * 2 - angle
-        arrow_size = 10.0 
-        arrow_p1 = end_point + PySide.QtCore.QPointF(
+        arrow_size = 10.0
+        arrow_p1 = end_point + QtCore.QPointF(
                 math.sin(angle - math.pi/3) * arrow_size,
                 math.cos(angle - math.pi/3) * arrow_size)
-        arrow_p2 = end_point + PySide.QtCore.QPointF(
+        arrow_p2 = end_point + QtCore.QPointF(
                 math.sin(angle - math.pi + math.pi/3) * arrow_size,
                 math.cos(angle - math.pi + math.pi/3) * arrow_size)
         path.lineTo(arrow_p1)
@@ -238,26 +297,43 @@ class Edge(genericSymbols.Connection, object):
         path.lineTo(arrow_p2)
         path.moveTo(end_point)
         try:
-            font = PySide.QtGui.QFont('arial', pointSize = 8)
-            width = PySide.QtGui.QFontMetrics(font).width(self.edge['label'])
+            font = QtGui.QFont('arial', pointSize=8)
+            width = QtGui.QFontMetrics(font).width(
+                    self.edge.get('label', 0))
             pos = self.mapFromScene(*self.edge['lp'])
-            path.addText(pos.x() - width/2, pos.y(),
-                    font, self.edge['label'])
+            #path.addText(pos.x() - width/2, pos.y(),
+            #        font, self.edge['label'])
+            if not self.text_label:
+                self.text_label = QtGui.QGraphicsTextItem(
+                                 self.edge.get('label', ''), parent=self)
+            self.text_label.setX(pos.x() - width / 2)
+            self.text_label.setY(pos.y())
+            self.text_label.setFont(font)
+            self.text_label.show()
         except KeyError:
             # no label
             pass
         self.setPath(path)
 
+
     def __str__(self):
         ''' user-friendly information about the edge coordinates '''
         return('Edge between ' + self.edge['source'].name + ' and ' +
                 self.edge['target'].name + str(self.edge['spline'][0]))
 
-    def paint(self, painter, _, ___):
+    def paint(self, painter, option, widget):
         ''' Apply anti-aliasing to Edge Connections '''
-        painter.setRenderHint(PySide.QtGui.QPainter.Antialiasing, True)
-        super(Edge, self).paint(painter, _, ___)
-
+        painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
+        super(Edge, self).paint(painter, option, widget)
+        # Draw lines between connection points, if visible
+        if self.bezier_visible:
+            painter.setPen(
+                    QtGui.QPen(QtCore.Qt.lightGray, 0, QtCore.Qt.SolidLine))
+            painter.setBrush(QtCore.Qt.NoBrush)
+            points_flat = [point.center
+                           for sub1 in self.bezier[1:] for point in sub1]
+            painter.drawPolyline([self.bezier[0]] + points_flat +
+                                 [self.end_point])
 
 def edges(scene, node):
     ''' Return all edges of a given node '''
@@ -429,9 +505,9 @@ def render_statechart(scene, graph=None, keep_pos=False):
 
     # Compute all the coordinates (self-modifying function)
     # Force the fontsize of the nodes to be 12, as in OpenGEODE
-    # use -n2 below to keep user-specified node coordinates 
-    graph.layout(prog='neato', args=
-            '-Nfontsize=12, -Gsplines=true -Gsep=1 -Gstart=random10 '
+    # use -n2 below to keep user-specified node coordinates
+    graph.layout(prog='neato', args='-Nfontsize=12, -Gsplines=true -Gsep=1 '
+            '-Gstart=random10 '
             '-Nstyle=rounded -Nshape=record -Elen=1.5 {kp}'
             .format(kp='-n1' if keep_pos else ''))
     # bb is not visible directly - extract it from the low level api:
