@@ -41,7 +41,7 @@
     For a complete example, look at the "sdlSymbols.py" module, that
     provide symbol definitions that correspond to an SDL editor.
 
-    Copyright (c) 2012 European Space Agency
+    Copyright (c) 2012-2013 European Space Agency
 
     Designed and implemented by Maxime Perrotin for the TASTE project
 
@@ -74,8 +74,12 @@ import ogParser
 
 if hasattr(sys, 'frozen'):
     # Detect py2exe distribution (that does not support the __file__ construct)
-    CUR_DIR = os.path.dirname(unicode
+    try:
+        CUR_DIR = os.path.dirname(unicode
             (sys.executable, sys.getfilesystemencoding()))
+    except TypeError:
+        # On Linux with pyinstaller the above will not work
+        CUR_DIR = os.path.dirname(os.path.realpath(__file__))
 else:
     CUR_DIR = os.path.dirname(os.path.realpath(__file__))
 
@@ -130,7 +134,7 @@ class Completer(QGraphicsProxyWidget, object):
     def keyPressEvent(self, e):
         super(Completer, self).keyPressEvent(e)
         if e.key() == Qt.Key_Escape:
-            self.parent.setFocus()
+            self.parentItem().setFocus()
         # Consume the event so that it is not repeated at EditableText level
         e.accept()
 
@@ -139,6 +143,7 @@ class Completer(QGraphicsProxyWidget, object):
         ''' When the user leaves the popup, return focus to parent '''
         super(Completer, self).focusOutEvent(event)
         self.hide()
+        self.resize(0, 0)
         self.parentItem().setFocus()
 
 
@@ -281,6 +286,8 @@ class EditableText(QGraphicsTextItem, object):
             invoked when selection is made
         '''
         text_cursor = self.textCursor()
+        # Go back to the previously saved cursor position
+        text_cursor.setPosition(self.cursor_position)
         extra = len(item.text()) - len(self.completion_prefix)
         if extra > 0:
             text_cursor.movePosition(QTextCursor.Left)
@@ -315,7 +322,9 @@ class EditableText(QGraphicsTextItem, object):
                 self.completion_prefix)
         if(completion_count > 0 and len(self.completion_prefix) > 1) or(
                 event.key() == Qt.Key_F8):
-            # Computing the position of the completer
+            # Save the position of the cursor
+            self.cursor_position = self.textCursor().position()
+            # Computing the coordinates of the completer
             # No direct Qt function for that.. doing it the hard way
             pos = self.textCursor().positionInBlock()
             block = self.textCursor().block()
@@ -329,11 +338,15 @@ class EditableText(QGraphicsTextItem, object):
 
             self.completer.setPos(pos_x, pos_y)
             self.completer.show()
+            # Make sure parent item has higher visibility than its siblings
+            # (useful in decision branches)
+            self.parentItem().setZValue(1)
             self.completer.setFocusProxy(self)
             self.setTabChangesFocus(True)
         else:
             self.completer.setFocusProxy(None)
             self.completer.hide()
+            self.completer.resize(0, 0)
             self.setFocus()
         self.completer_has_focus = False
 
@@ -350,34 +363,32 @@ class EditableText(QGraphicsTextItem, object):
             self.completer.hide()
             self.completer.resize(0, 0)
         if not self.completer or not self.completer.isVisible():
-            change = False
-            if self.oldSize != self.parentItem().boundingRect():
-                undo_cmd = undoCommands.ResizeSymbol(
-                                          self.parentItem(), self.oldSize,
-                                          self.parentItem().boundingRect())
-                self.scene().undo_stack.push(undo_cmd)
-                change = True
-                self.parentItem().cam(self.parentItem().pos(),
-                        self.parentItem().pos())
-            if self.oldText != str(self):
-                change = True
-                undo_cmd = undoCommands.ReplaceText(
-                                                 self, self.oldText, str(self))
-                self.scene().undo_stack.push(undo_cmd)
-            # End macro that was started in focusInEvent
-            self.scene().undo_stack.endMacro()
+            # Trigger a select - side effect makes the toolbar update
+            self.parentItem().select(True)
             self.editing = False
             text_cursor = self.textCursor()
             if text_cursor.hasSelection():
                 text_cursor.clearSelection()
                 self.setTextCursor(text_cursor)
-            # Do not repeat checks if no changes were made:
-            if change:
+            # If something has changed, check syntax and create undo command
+            if(self.oldSize != self.parentItem().boundingRect() or 
+                                                    self.oldText != str(self)):
                 # Call syntax checker from item containing the text (if any)
                 self.parentItem().check_syntax()
                 # Update class completion list
                 self.parentItem().update_completion_list()
-        self.setSelected(False)
+                # Create undo command, including possible CAM
+                with undoCommands.UndoMacro(self.scene().undo_stack, 'Text'):
+                    undo_cmd = undoCommands.ResizeSymbol(
+                                          self.parentItem(), self.oldSize,
+                                          self.parentItem().boundingRect())
+                    self.scene().undo_stack.push(undo_cmd)
+                    self.parentItem().cam(self.parentItem().pos(),
+                        self.parentItem().pos())
+
+                    undo_cmd = undoCommands.ReplaceText(
+                                               self, self.oldText, str(self))
+                    self.scene().undo_stack.push(undo_cmd)
         self.set_text_alignment()
         super(EditableText, self).focusOutEvent(event)
 
@@ -385,6 +396,8 @@ class EditableText(QGraphicsTextItem, object):
     def focusInEvent(self, event):
         ''' When user starts editing text, save previous state for Undo '''
         super(EditableText, self).focusInEvent(event)
+        # Trigger a select - side effect makes the toolbar update
+        self.parentItem().select(True)
         # Update completer list of keywords
         self.completer.set_completer_list()
         # Clear selection otherwise the "Delete" key may delete other items
@@ -392,7 +405,8 @@ class EditableText(QGraphicsTextItem, object):
         # Set width to auto-expand, and disables alignment, while editing:
         self.setTextWidth(-1)
         if not self.editing:
-            self.scene().undo_stack.beginMacro('Edit text')
+            #print 'edit text macro start'
+            #self.scene().undo_stack.beginMacro('Edit text')
             self.oldText = str(self)
             self.oldSize = self.parentItem().boundingRect()
             self.editing = True
@@ -413,6 +427,7 @@ class EditableText(QGraphicsTextItem, object):
 # pylint: disable=R0904
 class Connection(QGraphicsPathItem, object):
     ''' Connection between two symbols (arrow) '''
+    # Default connections are not selectable
     default_cursor = Qt.ArrowCursor
 
     def __init__(self, parent, child, connectionPoint=False):
@@ -458,8 +473,6 @@ class Connection(QGraphicsPathItem, object):
         if self.child_is_a_connection_point:
             connection_point_scene = self.child.mapToScene(
                     self.child.connectionPoint)
-            #connection_point_local = self.parentItem().mapFromScene(
-            #        connection_point_scene)
             connection_point_local = self.mapFromScene(
                     connection_point_scene)
             self.end_point = connection_point_local
@@ -475,12 +488,11 @@ class Connection(QGraphicsPathItem, object):
                         self.child.boundingRect().height() / 2)
         else:
             self.end_point = QPointF()
-            #self.end_point.setY(self.child.pos().y())
+            self.end_point.setY(self.child.pos().y())
             # FIXME - why is the pos of the child above the connection point
-            # when loading?!
-            self.end_point.setY(
-                    max(self.child.pos().y(), self.start_point.y()))
-            #print self.child.pos(), self.start_point, self.end_point
+            # when loading?! MP: fixed
+        #   self.end_point.setY(
+        #           max(self.child.pos().y(), self.start_point.y()))
         # Move to start point and draw the connection
         new_shape.moveTo(self.start_point)
         if not self.child_is_a_connection_point and not isinstance(
@@ -647,6 +659,19 @@ class Symbol(QObject, QGraphicsPathItem, object):
             self.common_name, repr(self))
         return ast, terminators
 
+    def edit_text(self, pos=None):
+        '''
+            Set the focus on the text area for edition. Position is optional
+            Can be called when user double clicks on an item
+            Position is in scene coordinates
+        '''
+        _ = pos
+        try:
+            # Only one text area is supported for now - ignoring position
+            self.text.setFocus()
+        except AttributeError:
+            return
+
     def update_completion_list(self):
         '''
             Update the text autocompletion list based on the symbol text
@@ -963,20 +988,19 @@ class Symbol(QObject, QGraphicsPathItem, object):
     def mouse_release(self, _):
         ''' Default action when mouse is released: reset mode '''
         if self.mode == 'Resize' and self.old_rect != self.boundingRect():
-            self.scene().undo_stack.beginMacro('Resize symbol')
-            undo_cmd = undoCommands.ResizeSymbol(
-                                       self, self.old_rect,
-                                       self.boundingRect())
-            self.scene().undo_stack.push(undo_cmd)
-            self.cam(self.coord, self.pos())
-            self.updateConnectionPoints()
-            self.scene().undo_stack.endMacro()
+            with undoCommands.UndoMacro(self.scene().undo_stack, 'Resize'):
+                undo_cmd = undoCommands.ResizeSymbol(
+                                           self, self.old_rect,
+                                           self.boundingRect())
+                self.scene().undo_stack.push(undo_cmd)
+                self.cam(self.coord, self.pos())
+                self.updateConnectionPoints()
         elif self.mode == 'Move' and self.coord != self.pos():
-            self.scene().undo_stack.beginMacro('Move symbol')
-            undo_cmd = undoCommands.MoveSymbol(self, self.coord, self.pos())
-            self.scene().undo_stack.push(undo_cmd)
-            self.cam(self.coord, self.pos())
-            self.scene().undo_stack.endMacro()
+            with undoCommands.UndoMacro(self.scene().undo_stack, 'Move'):
+                undo_cmd = undoCommands.MoveSymbol(
+                                            self, self.coord, self.pos())
+                self.scene().undo_stack.push(undo_cmd)
+                self.cam(self.coord, self.pos())
         self.mode = ''
 
     def updateConnectionPoints(self):
@@ -1121,7 +1145,6 @@ class Comment(Symbol, object):
                 hyperlink=ast.hyperlink)
         self.common_name = 'end'
         self.parser = ogParser
-        #self.grabber.display()
 
     def __str__(self):
         return 'Comment'
@@ -1496,7 +1519,7 @@ class VerticalSymbol(Symbol, object):
         In case of SDL: STATE, OUTPUT, PROCEDURE, DECISION, TASK
     '''
     def __init__(self, parent=None, text='...',
-            y=None, hyperlink=None):
+            x=None, y=None, hyperlink=None):
         super(VerticalSymbol, self).__init__(parent)
         self.connection = None
         self.text = EditableText(parent=self, text=text, hyperlink=hyperlink)
@@ -1504,6 +1527,8 @@ class VerticalSymbol(Symbol, object):
         if parent:
             local_pos = self.mapFromScene(0, y or 0)
             self.insert_symbol(parent=parent, x=None, y=local_pos.y())
+        else:
+            self.setPos(x or 0, y or 0)
 
     def next_aligned_symbol(self):
         ''' Return the next symbol in the flow '''
@@ -1631,13 +1656,3 @@ class VerticalSymbol(Symbol, object):
             branch_entry.cam(branch_entry.pos(), branch_entry.pos())
         else:
             super(VerticalSymbol, self).cam(old_pos, new_pos)
-
-#    def paint(self, painter, _, ___):
-       # Removed MP 02/05/2013 - Not needed, positions are computed properly
-       #       '''
-       #           Make sure symbol is V-aligned below its parent
-       #           and that it is not colliding with it
-       #       '''
-#       if self.hasParent:
-#           self.update_position()
-#       super(VerticalSymbol, self).paint(painter, _, ___)
