@@ -27,7 +27,7 @@
 
     See AdaGenerator.py for an example of use.
 
-    Copyright (c) 2012 European Space Agency
+    Copyright (c) 2012-2013 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -56,12 +56,24 @@ class Expression(object):
         self.kind = None
         # var is of type Primary
         self.var = None
+        # Hint for code generators: intermediate storage identifier
+        self.tmpVar = -1
 
     def __repr__(self):
         ''' Debug output for an expression '''
         return '{k}: {exp} ({l},{c})'.format(exp=self.inputString,
                 k=self.kind.upper(), l=self.line, c=self.charPositionInLine)
 
+    def is_raw(self):
+        ''' Helper function - check if an expression is a terminal primary '''
+        return (self.kind == 'primary' and self.var.kind != 'expression'
+                and (self.var.kind != 'primaryId' or
+                    (len(self.var.primaryId) < 2
+                     and self.exprType['Kind'] != 'ReferenceType')))
+#       return (self.kind == 'primary' and self.var.kind != 'expression'
+#               and len(self.var.primaryId) < 2
+#               and self.exprType['Kind'] != 'ReferenceType')
+
 
 class Primary(object):
     ''' AST entry for a primary construction '''
@@ -98,7 +110,7 @@ class Primary(object):
         # to process the ifThenElse
         self.ifThenElse = None
         # stringLiteral is a string (in e.g. a := 'Hello',
-        #    'Hello' is the string literal)
+        #    'Hello' is the string literal. Quotes are kept)
         self.stringLiteral = None
         # mantissaBaseExpFloat is a dict:
         # {'mantissa': int, 'base': int, 'exponent': int}
@@ -109,6 +121,8 @@ class Primary(object):
         self.sequenceOf = []
         # sequence is a dict : { field1: Expression, field2: Expression }
         self.sequence = {}
+        # Hint for code generators: intermediate storage identifier
+        self.tmpVar = -1
 
     def __repr__(self):
         ''' Debug output for a primary '''
@@ -281,6 +295,8 @@ class Label(object):
         self.hyperlink = None
         # No comment for labels - keep to None
         self.comment = None
+        # List of terminators following this label
+        self.terminators = []
 
     def __repr__(self):
         ''' Debug output for a label '''
@@ -288,6 +304,20 @@ class Label(object):
                 l=self.line, c=self.charPositionInLine)
 
 
+class Floating_label(Label, object):
+    ''' AST Entry for a floating label '''
+    def __init__(self):
+        ''' Initialize the floating label attributes '''
+        super(Floating_label, self).__init__()
+        # Transition following the label - other attribues inherited from Label
+        self.transition = None
+
+    def __repr__(self):
+        ''' Debug output for a label (used by code generator) '''
+        return 'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
+                l=self.line, c=self.charPositionInLine)
+
+
 class Transition(object):
     ''' AST Entry for a complete transition '''
     def __init__(self):
@@ -296,27 +326,8 @@ class Transition(object):
         self.actions = []
         # terminator is of type Terminator (it is optional)
         self.terminator = None
-
-
-class Floating_label(object):
-    ''' AST Entry for a floating label '''
-    def __init__(self):
-        ''' Initialize the floatinglabel attributes '''
-        self.inputString = 'Here'
-        # optional hyperlink
-        self.hyperlink = None
-        self.pos_x, self.pos_y = 0, 0
-        self.width, self.height = 100, 50
-        self.line = None
-        self.charPositionInLine = None
-        self.transition = None
-        # list of terminators
-        self.terminators = []    
-    
-    def __repr__(self):
-        ''' Debug output for a label (used by code generator) '''
-        return 'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
-                l=self.line, c=self.charPositionInLine)
+        # All Terminators of this transition
+        self.terminators = []
 
 
 class Input(object):
@@ -377,6 +388,11 @@ class Start(object):
         return 'START'
 
 
+class Procedure_start(Start):
+    ''' Procedure start symbol - inherits from Start '''
+    pass
+
+
 class Comment(object):
     ''' AST Entry for COMMENT symbols '''
     def __init__(self):
@@ -449,6 +465,18 @@ class TextArea(object):
                 l=self.line, c=self.charPositionInLine)
 
 
+class Automaton(object):
+    ''' Elements contained in a process or a procedure '''
+    def __init__(self, parent=None):
+        ''' AST grouping the elements that can be rendered graphically '''
+        self.parent = parent
+        self.textAreas = []
+        self.inner_procedures = []
+        self.start = None
+        self.floating_labels = []
+        self.states = []
+
+
 class Procedure(object):
     ''' Internal procedure definition '''
     def __init__(self):
@@ -470,16 +498,18 @@ class Procedure(object):
         # [{'name': str, 'direction': 'in'/'out', 'type': str}]
         self.fpar = []
         # start, states, terminators, text areas, floating_labels (see Process)
-        self.start = None
-        self.states = []
+        #self.start = None
+        #self.states = []
         self.terminators = []
-        self.textAreas = []
-        self.floating_labels = []
+        #self.textAreas = []
+        # Keep a list of labels and floating labels - useful for backends
+        self.labels = []
+        #self.floating_labels = []
         # Inherited procedures and operators
         self.procedures = {}
         self.operators = {}
         # inner procedures and operators (see Process for format)
-        self.inner_procedures = []
+        #self.inner_procedures = []
         self.inner_operators = []
         # mapping, transitions: see Process
         self.mapping = {}
@@ -488,6 +518,8 @@ class Procedure(object):
         self.external = False
         # Optional comment
         self.comment = None
+        # Set of symbols contained in the procedure (type Automaton)
+        self.content = Automaton(parent=self)
 
 
 class Process(object):
@@ -497,7 +529,7 @@ class Process(object):
         self.processName = None
         # Optional filename containing this process (PR file)
         self.filename = None
-        # Process parent context (BLOCK)
+        # Process parent context (Block or AST if defined at root level)
         self.parent = None
         # A process can be referenced (externally defined)
         self.referenced = False
@@ -509,7 +541,7 @@ class Process(object):
 
         # dataView: complete AST of the ASN.1 types
         self.asn1Modules = None
-        self.dataView = None
+        self.dataview = None
 
         # input and output signal lists: {'signal': definition from iv.py, ...}
         # input and output signal lists:
@@ -518,29 +550,26 @@ class Process(object):
         self.output_signals = []
 
         # start, states, and terminators are useful for rendering backends
-        self.start = None
-        self.states = []
+#        self.start = None
+#        self.states = []
         self.terminators = []
 
+        # Keep a list of labels and floating labels - useful for backends
+        # List of Label
+        self.labels = []
         # List of Floating_label
-        self.floating_labels = []
+#        self.floating_labels = []
 
         # list of operators (not supported) and procedures
         self.operators = {}
-        # procedures: same form as outputlist: NOT ANYMORE
-        #     {'procedureName': definition from iv.py, ...}
-        # only supports procedures as sync RI specified in the
-        # TASTE interface view
-        # use operators or inner_procedures for other purposes
-        #self.procedures = {}
-        # list of class Procedure (external procedures)
+        # list of Procedure (external procedures)
         self.procedures = []
         # User-defined procedures: list of ogAST.Procedure
-        self.inner_procedures = []
+#        self.inner_procedures = []
 
         # list of type TextArea. Useful for rendering only
         # use 'variables', 'operators' and 'procedures' to get useful content
-        self.textAreas = []
+#        self.textAreas = []
 
         # The Mapping structure should be used for code generation backends
         # dictionnary: {'stateName': [class Input1, Input2,...], ...}
@@ -549,6 +578,9 @@ class Process(object):
 
         # list of type Transition - use 'mapping' to map index to inputs/states
         self.transitions = []
+        
+        # Set of symbols contained in the process (type Automaton)
+        self.content = Automaton(parent=self)
 
 
 class Block(object):
@@ -585,6 +617,10 @@ class System(object):
         '''
         # System name
         self.name = ''
+        # Optional filename containing this system (PR file)
+        self.filename = None
+        # Reference to top-level AST
+        self.ast = None
         # list of SIGNAL declarations: [{'name': str, 'type': str}]
         # (Supporting only one parameter)
         self.signals = []
