@@ -10,7 +10,7 @@
     All symbols inherit the generic Vertical- and Horizontal-
     Symbol classes defined in the "genericSymbols.py" module.
 
-    Copyright (c) 2012 European Space Agency
+    Copyright (c) 2012-2013 European Space Agency
 
     Designed and implemented by Maxime Perrotin
 
@@ -47,7 +47,7 @@ SDL_BLACKBOLD = ['\\b{word}\\b'.format(word=word) for word in (
 
 SDL_REDBOLD = ['\\b{word}\\b'.format(word=word) for word in (
               'INPUT', 'OUTPUT', 'STATE', 'DECISION', 'NEXTSTATE',
-              'TASK', 'TIMER', 'PROCESS', 'LABEL', 'JOIN')]
+              'TASK', 'TIMER', 'PROCESS', 'LABEL', 'JOIN', 'CONNECTION')]
 
 
 # pylint: disable=R0904
@@ -57,6 +57,7 @@ class Input(HorizontalSymbol, object):
     _insertable_followers = ['Task', 'ProcedureCall', 'Output', 'Decision',
                         'Input',  'Label']
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
+    completion_list = set()
 
     common_name = 'input_part'
     # Define reserved keywords for the syntax highlighter
@@ -146,13 +147,15 @@ class Output(VerticalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Output()
         super(Output, self).__init__(parent=parent,
-                text=ast.inputString, y=ast.pos_y, hyperlink=ast.hyperlink)
+                text=ast.inputString, x=ast.pos_x, y=ast.pos_y,
+                hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
-        
+
         self.setBrush(QBrush(QColor(255, 255, 202)))
         self.terminal_symbol = False
         self.parser = ogParser
@@ -199,13 +202,14 @@ class Decision(VerticalSymbol, object):
     blackbold = SDL_BLACKBOLD + ['\\b{}\\b'.format(word)
                                    for word in ('AND', 'OR')]
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Decision()
         # Define the point where all branches of the decision can join again
         self.connectionPoint = QPoint(ast.width / 2, ast.height + 30)
         super(Decision, self).__init__(parent, text=ast.inputString,
-                y=ast.pos_y, hyperlink=ast.hyperlink)
+                x=ast.pos_x, y=ast.pos_y, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setBrush(QColor(255, 255, 202))
         self.minDistanceToSymbolAbove = 0
@@ -330,6 +334,7 @@ class Decision(VerticalSymbol, object):
                 child.moveBy(0, delta)
             except AttributeError:
                 pass
+        self.update_connections()
 
     def pr(self, recursive=True):
         ''' Get PR notation of a decision (possibly recursively) '''
@@ -366,6 +371,7 @@ class DecisionAnswer(HorizontalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Answer()
@@ -412,9 +418,7 @@ class DecisionAnswer(HorizontalSymbol, object):
 
     def set_shape(self, width, height):
         ''' ANSWER has round, disjoint sides - does not fit in a polygon '''
-        #print str(self), width, height
         self.width, self.height = width, height
-        #traceback.print_stack()
         path = QPainterPath()
         left = QRect(0, 0, 35, height)
         right = QRect(width - 35, 0, 35, height)
@@ -423,23 +427,9 @@ class DecisionAnswer(HorizontalSymbol, object):
         path.arcMoveTo(right, -55)
         path.arcTo(right, -55, 110)
         path.moveTo(width, height)
-
-        #print path.boundingRect()
-        #print path.controlPointRect()
-
         self.setPath(path)
         super(DecisionAnswer, self).set_shape(width, height)
 
-#   def paint(self, painter, _, __):
-#       ''' Draw ANSWER shape within a polygon boundaries '''
-#       rect = self.boundingRect()
-#       painter.setRenderHint(QPainter.Antialiasing, True)
-#       left = QRect(rect.x(), rect.y(), 35, rect.height())
-#       right = QRect(rect.width() - 35, rect.y(), 35, rect.height())
-#       painter.drawArc(left, 2000, 1760)
-#       painter.drawArc(right, -880, 1760)
-#       super(DecisionAnswer, self).paint(painter, _, __)
-
     def pr(self, recursive=True):
         ''' Return the PR string for the symbol, possibly recursively '''
         ans = str(self)
@@ -473,6 +463,7 @@ class Join(VerticalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         if not ast:
@@ -481,7 +472,7 @@ class Join(VerticalSymbol, object):
             ast.width = 50
             ast.height = 50
         super(Join, self).__init__(parent, text=ast.inputString,
-                y=ast.pos_y, hyperlink=ast.hyperlink)
+                x=ast.pos_x, y=ast.pos_y, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setPen(QPen(Qt.blue))
         self.terminal_symbol = True
@@ -530,6 +521,7 @@ class ProcedureStop(Join, object):
             ast.width = 50
             ast.height = 50
         super(ProcedureStop, self).__init__(parent, ast)
+    completion_list = set()
 
     def set_shape(self, width, height):
         ''' Define the symbol shape '''
@@ -572,11 +564,12 @@ class Label(VerticalSymbol, object):
     redbold = SDL_REDBOLD
     # Symbol must not use antialiasing, otherwise the middle line is too thick
     _antialiasing = False
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Label()
         super(Label, self).__init__(parent, text=ast.inputString,
-                y=ast.pos_y, hyperlink=ast.hyperlink)
+                x=ast.pos_x, y=ast.pos_y, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setPen(QPen(Qt.blue))
         self.terminal_symbol = False
@@ -649,14 +642,15 @@ class Task(VerticalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ''' Initializes the TASK symbol '''
         ast = ast or ogAST.Task()
         super(Task, self).__init__(parent, text=ast.inputString,
-                y=ast.pos_y, hyperlink=ast.hyperlink)
+                x=ast.pos_x, y=ast.pos_y, hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
-        
+
         self.setBrush(QBrush(QColor(255, 255, 202)))
         self.terminal_symbol = False
         self.parser = ogParser
@@ -701,11 +695,13 @@ class ProcedureCall(VerticalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.Output(defName='callMe')
         super(ProcedureCall, self).__init__(parent,
-                text=ast.inputString, y=ast.pos_y, hyperlink=ast.hyperlink)
+                text=ast.inputString, x=ast.pos_x, y=ast.pos_y,
+                hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setBrush(QBrush(QColor(255, 255, 202)))
         self.terminal_symbol = False
@@ -749,6 +745,7 @@ class TextSymbol(HorizontalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, ast=None):
         ''' Create a Text Symbol '''
@@ -822,12 +819,14 @@ class State(VerticalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, parent=None, ast=None):
         ast = ast or ogAST.State()
         # Note: ast coordinates are in scene coordinates
         super(State, self).__init__(parent=parent,
-                text=ast.inputString, y=ast.pos_y, hyperlink=ast.hyperlink)
+                text=ast.inputString, x=ast.pos_x, y=ast.pos_y,
+                hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
         self.setBrush(QBrush(QColor(255, 228, 213)))
         self.terminal_symbol = True
@@ -935,6 +934,7 @@ class Procedure(HorizontalSymbol, object):
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    completion_list = set()
 
     def __init__(self, ast=None, subscene=None):
         ast = ast or ogAST.Procedure()
@@ -1050,6 +1050,7 @@ class Start(HorizontalSymbol, object):
             next_symbol = next_symbol.next_aligned_symbol()
         return '\n'.join(result)
 
+
 class ProcedureStart(Start, object):
     ''' Start symbol of a procedure - only shape differs from Start '''
     # Define reserved keywords for the syntax highlighter
